using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GenerateHeightMap
{
    public partial class Form1 : Form
    {
        //Lagring fr vr horisentallinje
        double[,] HeightMap = new double[513, 513];
        //Randomerare fr slumptal
        private Random rnd = new Random();
        //Bilden som kommer attt genereras i form av en bitmap
        Bitmap b = new Bitmap(513, 513);

        public Form1()
        {
            InitializeComponent();

            //Frglgger bilden svart vid start
            //
            Graphics g = Graphics.FromImage(b);
            g.Clear(Color.Black);
            pictureBox1.Image = b;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Skapa om data och bild beroende p genererad storlek;
            int size = (int)Math.Pow(2, (double)numericUpDown1.Value);
            b = new Bitmap(size + 1, size + 1);
            HeightMap = new double[size + 1, size + 1];
            pictureBox1.Size = new Size(size + 1, size + 1);

            //Brjar algoritmen med slump +- 1.0
            double Max = 1.0;
            //Bestmmer ojmnheten i terrngen
            double Roughness = ((double)numericUpDown2.Value / 100.0);

            //Slumpa startvrden i alla hrn
            HeightMap[0, 0] = (rnd.NextDouble() * 2.0 - 1.0) * Max;
            HeightMap[size, 0] = (rnd.NextDouble() * 2.0 - 1.0) * Max;
            HeightMap[0, size] = (rnd.NextDouble() * 2.0 - 1.0) * Max;
            HeightMap[size, size] = (rnd.NextDouble() * 2.0 - 1.0) * Max;

            //Utfr ett antal iterationer beroende p bildens storlek
            for (int start = size / 2; start >= 1; start = start / 2)
            {
                for (int y = start; y < size; y += 2 * start)
                {
                    for (int x = start; x < size; x += 2 * start)
                    {
                        //Mittpunkt i rektangeln
                        double tmp = (HeightMap[x - start, y - start] + HeightMap[x + start, y + start] +
                            HeightMap[x + start, y - start] + HeightMap[x - start, y + start]) / 4.0;
                        //Lgg till en slumpvis frskjutning +-Max
                        tmp += (rnd.NextDouble() * 2.0 - 1.0) * Max;
                        //Uppdatera punkt i kartan
                        HeightMap[x, y] = tmp;

                        //Berkna punkt ovanfr
                        HeightMap[x, y - start] = (HeightMap[x - start, y - start] + HeightMap[x + start, y - start]) / 2.0;
                        HeightMap[x, y - start] += (rnd.NextDouble() * 2.0 - 1.0) * Max;
                        //Berkna punkt till vnster
                        HeightMap[x - start, y] = (HeightMap[x - start, y - start] + HeightMap[x - start, y + start]) / 2.0;
                        HeightMap[x - start, y] += (rnd.NextDouble() * 2.0 - 1.0) * Max;
                        //Berkna punkt till hger
                        HeightMap[x + start, y] = (HeightMap[x + start, y - start] + HeightMap[x + start, y + start]) / 2.0;
                        HeightMap[x + start, y] += (rnd.NextDouble() * 2.0 - 1.0) * Max;
                        //Berkna punkt nedanfr
                        HeightMap[x, y + start] = (HeightMap[x - start, y + start] + HeightMap[x + start, y + start]) / 2.0;
                        HeightMap[x, y + start] += (rnd.NextDouble() * 2.0 - 1.0) * Max;
                    }
                }
                //Berkna Max fr nsta iteration, avtagande med en faktor 2^(x)
                Max = Max * Math.Pow(2, -Roughness);
            }

            //Rendera grskalig bild
            for (int y = 0; y < HeightMap.GetUpperBound(0); y++)
            {
                for (int x = 0; x < HeightMap.GetUpperBound(1); x++)
                {
                    //Vrden i HeightMap ligger i runda slngar +- 1.0
                    //Skala med 127 ger +-127 och lgg till offset p 128 ger 0 till 255
                    int greyscale = (int)((HeightMap[x, y] * 127.0) + 128.0);

                    //Extra kolla s att vrdena ligger inom giltiga grnser
                    if (greyscale < 0)
                        greyscale = 0;
                    if (greyscale > 255)
                        greyscale = 255;
                    //Uppdatera pixel med grskaligt vrde
                    b.SetPixel(x, y, Color.FromArgb(greyscale, greyscale, greyscale));
                }
            }
            //Uppdatera bilden
            pictureBox1.Image = b;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                pictureBox1.Image.Save(saveFileDialog1.FileName); 
            }
        }
    }
}